var http = require('http');
var dgram = require('dgram');
var util = require('util');
var querystring = require('querystring');
var EventEmitter = require('events').EventEmitter;
var Counter = require('metrics').Counter;
var Histogram = require('metrics').Histogram;

var DEFAULT_BROKER_SOCK = require('./defaults').DEFAULT_BROKER_SOCK;
var DEFAULT_SERVER_PORT = require('./defaults').DEFAULT_SERVER_PORT;

// JavaScript uses double-precision FP for all numeric types.  
// Perhaps someday we'll have native 64-bit integers that can safely be
// transported via JSON without additional code, but not today.
var MAX_COUNTER_VALUE = Math.pow(2, 32); // 4294967296

var types = exports.types = {
  COUNTER: 'C',
  HISTOGRAM: 'H',
};


/**
 * Handle counter operation.
 *
 * @param {String} op A counter operation (INC, DEC, or CLEAR).
 * @param {String} name Metric name.
 * @param {String} dimensions Metric dimensions (composed as a query string).
 * @param {Array} args Operational arguments (for INC and DEC, the amount to 
 *  increment or decrement the counter by).
 */
function handleCounterOp(op, name, dimensions, val) {
  var counter;
  
  counter = this.metrics[dimensions][name];
  if (! counter) {
    counter = new Counter();
    this.metrics[dimensions][name] = counter;
  } 
  if (counter instanceof Counter) {
    if (val === undefined || val === null) {
      val = 1;
    } else if (typeof val === 'string') {
      val = parseInt(val, 10);
    }
    switch(op) {
      case 'INC':
        counter.inc(val);
        break;
      case 'DEC':
        counter.dec(val);
        break;
      case 'CLEAR':
        counter.clear();
        break;
    }
  } else {
    throw new Error(name + ' is not a counter');
  }
};


/**
 * Handle histogram operation.
 *
 * @param {String} op A histogram operation (UPDATE or CLEAR).
 * @param {String} name Metric name.
 * @param {String} dimensions Metric dimensions (composed as a query string).
 * @param {Array} args Operational arguments (for UPDATE, the value to set).
 */
function handleHistogramOp(op, name, dimensions, val) {
  var histogram;
  
  histogram = this.metrics[dimensions][name];
  if (! histogram) {
    histogram = new Histogram();
    this.metrics[dimensions][name] = histogram;
  } 
  if (histogram instanceof Histogram) {
    if (typeof val === 'string') {
      val = parseFloat(val);
    }
    switch(op) {
      case 'UPDATE':
        histogram.update(val || 0);
        break;
      case 'CLEAR':
        histogram.clear();
        break;
    }
  } else {
    throw new Error(name + ' is not a histogram');
  }
};


/**
 * Clear all metrics.
 */
function clearMetrics() {
  this.metrics = {};
}


/**
 * Broker request handler; not to be called externally.
 *
 * @param {String} msg The incoming request message.
 */
function handleBrokerRequest(msg) {
  var msg, i, keys, normalized_dimensions;

  try {
    msg = JSON.parse(msg);
  } catch(e) {
    throw new Error('Failed to parse message "' + msg + '": bad JSON?');
  }
  
  if (msg.dimensions === undefined || msg.dimensions === null) {
    msg.dimensions = {};
  }

  // Special case: CLEAR operation
  if (msg.op === 'CLEAR' && ! msg.type) {
    clearMetrics.call(this);
    return;
  }

  // Normalize dimensions.
  normalized_dimensions = {};
  keys = Object.keys(msg.dimensions).sort();
  for (i = 0; i < keys.length; i++) {
    normalized_dimensions[keys[i]] = msg.dimensions[keys[i]];
  }
  normalized_dimensions = querystring.stringify(normalized_dimensions);

  if (! (normalized_dimensions in this.metrics)) {
    this.metrics[normalized_dimensions] = {};
  }

  var handlers = {
    C: handleCounterOp,
    H: handleHistogramOp,
  };

  if (msg.type in handlers) {
    handlers[msg.type].call(this, msg.op, msg.name, normalized_dimensions, msg.val);
  }
};


function handleHTTPRequest(req, res) {
  var key,
      metric,
      result,
      count = 0;

  if (req.url.match('^/metrics')) {
    res.writeHead(200, {'Content-Type': 'application/json'});
    for (key in this.metrics) {
      result = {};

      if (key) {
        result.dimensions = querystring.parse(key);
      }
      for (metric in this.metrics[key]) {
        result[metric] = this.metrics[key][metric].printObj();
      }
      res.write(JSON.stringify(result) + '\n', 'utf8');
    }
    res.end();
  } else {
    res.writeHead(404, {'Content-Type': 'text/plain'});
    res.end('ERROR: Invalid path.  Metrics can be retrieved from /metrics.\n');
  }
};

/**
 * Create a metrics broker instance.
 *
 * @constructor
 * @param {Object} args A set of keyword arguments used to configure the broker.  
 *  'brokerPath' is the UNIX domain socket path to which clients should send
 *  messages (default: /tmp/node_metrics_broker). 
 *  'serverPort' is the TCP port number or UNIX domain socket path for the 
 *  HTTP server to which monitoring agents should connect.  'serverHost' is the
 *  interface the HTTP server should listen on (if unspecified, INADDR_ANY).
 */
var MetricsBroker = exports.MetricsBroker = function MetricsBroker(args) {
  var self = this;

  args = args || {};

  if (! (self instanceof MetricsBroker)) {
    return new MetricsBroker(args);
  }

  self.metrics = {};
  
  // Initialize broker
  self.broker = dgram.createSocket('unix_dgram');
  self.broker.bind(args.brokerPath || DEFAULT_BROKER_SOCK);
  self.broker.on('listening', function() { self.emit('brokerReady'); });
  self.broker.on('message', function(msg, rinfo) {
    try { 
      msg = msg.toString('utf8');
      handleBrokerRequest.call(self, msg); 
    }
    catch (e) { console.error(e.name + ": " + e.message); }
  });
  self.broker.on('close', function(e) { self.emit('brokerClose', e); });

  // Initialize server
  self.server = http.createServer();
  self.server.listen(args.serverPort || DEFAULT_SERVER_PORT, args.serverHost,
      function() { self.emit('serverReady'); });
  self.server.on('request',
      function(req, res) { handleHTTPRequest.call(self, req, res); });
  self.broker.on('error', function(e) { self.emit('serverError', e); });
  self.broker.on('close', function(e) { self.emit('serverClose', e); });

  return self;
};

util.inherits(MetricsBroker, EventEmitter);

MetricsBroker.prototype.shutDown = function() {
  this.broker.close();
  this.server.close();
};
