/**
 * Client for metrics broker
 *
 * This module communicates with the metrics broker over a UNIX datagram
 * socket.
 */

var dgram = require('dgram');
var brokerPath = require('./defaults').DEFAULT_BROKER_SOCK;
var client = dgram.createSocket('unix_dgram');

/**
 * Sets the UNIX domain socket path for broker communication.  Don't call
 * this unless the default value causes a problem.
 *
 * @param {String} path The path; must be the same one the broker is listening
 * to.
 */
exports.setPath = function setPath(path) {
  brokerPath = path;
}


/**
 * Send a metrics manipulation message to the broker.
 *
 * @param {String} type Metric type.
 * @param {String} op Operation.
 * @param {String} name Metric name.
 * @param {Object} dimensions Metric dimensions.
 * @param {Number} val Metric value.
 * @param {Function(err)} callback An optional callback (usually omitted) to
 *  be called after the message has been dispatched.
 */
exports.send = function send(type, op, name, dimensions, val, callback) {
  var msg = new Buffer(JSON.stringify({
    type: type,
    op: op,
    name: name,
    dimensions: dimensions,
    val: val
  }), 'utf8');

  if (typeof callback === 'function') {
    client.send(msg, 0, msg.length, brokerPath, 
                function(err, bytes) { callback(err); });
  } else {
    client.send(msg, 0, msg.length, brokerPath);
  }
};


/**
 * Send a CLEAR message to the broker.  This is highly destructive and will
 * evict all metrics from the broker; use with caution.
 *
 * @param {Function(err)} callback An optional callback (usually omitted) to
 *  be called after the message has been dispatched.
 */
exports.clear = function clear(callback) {
  var msg = new Buffer(JSON.stringify({ op: 'CLEAR' }), 'utf8');

  if (typeof callback === 'function') {
    client.send(msg, 0, msg.length, brokerPath, 
                function(err, bytes) { callback(err); });
  } else {
    client.send(msg, 0, msg.length, brokerPath);
  }
};
