/**
 * This file implements basic histogram functionality for JavaScript programs
 * that use the metrics broker.
 *
 * Example usage:
 *
 * require('metrics-broker/instruments').Histogram;
 *
 * var name = 'actions';
 * var dimensions = { cust: 'custA', ... };
 * Histogram.update(name, dimensions, val, [callback]);
 *
 * In most cases, callback should be omitted.
 *
 * See the README for more information on names and dimensions.
 */

var types = require('../broker').types;
var Client = require('../broker').Client;


/** 
 * Adds a new measurement to the specified histogram in the broker.
 *
 * @param {String} name The name of the histogram.
 * @param {Object} dimensions The dimensions of the histogram.  May be null, if
 *  no dimensions are desired.
 * @param {Number} val Value to be added to the histogram.
 * @param {Function(String)} callback A callback to be invoked after the
 *  message has been sent.  Most code can safely omit this.
 */
exports.update = function update(name, dimensions, val, callback) {
  if (! name) {
    throw new Error('No name specified');
  }
  if (dimensions && (typeof dimensions !== 'object')) {
    throw new Error('Dimensions must be an object');
  }
  Client.send(types.HISTOGRAM, 'UPDATE', name, dimensions, val, callback);
};

exports.set = exports.update;


/** 
 * Clears the specified histogram in the broker.
 *
 * @param {String} name The name of the histogram.
 * @param {Object} dimensions The dimensions of the histogram.  May be null, if
 *  no dimensions are desired.
 * @param {Function(String)} callback A callback to be invoked after the
 *  message has been sent.  Most code can safely omit this.
 */
exports.clear = function clear(name, dimensions, callback) {
  if (! name) {
    throw new Error('No name specified');
  }
  if (dimensions && (typeof dimensions !== 'object')) {
    throw new Error('Dimensions must be an object');
  }
  Client.send(types.HISTOGRAM, 'CLEAR', name, dimensions, null, callback);
};
